@@ -2,5 +2,7 @@ from ._cma import CMA  # NOQA
 from ._sepcma import SepCMA  # NOQA
 from ._warm_start import get_warm_start_mgd  # NOQA
 from ._cmawm import CMAwM  # NOQA
+from ._xnes import XNES  # NOQA
+from ._dxnesic import DXNESIC  # NOQA
 
-__version__ = "0.9.1"
+__version__ = "0.10.0"
